local InfoMsg=true;			-- Is info messages shown
local DebugMsg=false;		-- Is debug messages shown
local Loaded=false;			-- Is addon loaded
local ForceVisible=false;	-- If bar is forced to be shown

-- Spell data
local SpellName=nil;		-- Spell name, set to nil to disable
local SpellRank=0;			-- Spell rank
local SpellTime=0;			-- Spell cast time
local SpellHealAmount=1;	-- Suspected healing amount
local SpellTarget=nil;		-- Spell target
local SpellTargetName=nil;	-- Target's name
local UpdateHealth=false;	-- Is target's health changed
local OverhealDetected=false;	-- If spell detected to do overhealing

-- Party data
local GroupMaxHeal={};		-- How much each party member can receive healing, sorted by name
local AquiredTarget=nil;		-- Found spell target

-- Healing meter
local TotalHealing=0;		-- Total healing
local TotalOverHealing=0;	-- Total overhealing
local TotalCritHealing=0;   -- Total healing from crits
local TotalCritOverHealing=0;   -- Total overhealing from crits

-- Bar width
local barwidth=150;

HEChartSpellName = "xx";

-- Saved variables
HealingEstimatorData=
	{
	Scale=1;					-- Healing bar scale
	Limit=0.5;					-- Overhealing alert limit
	Icon=true;					-- If minimap icon is shown
	IconPos=4.98;				-- Position in radians
	IconAnchoredAtMinimap=true;	-- If icon is anchored to minimap
	ShowOverhealPercent=false;
	ShowHealthDeficit=true;
	ShowHealOverTimes=true;
	ShowPortrait=false;
	ShowSpellTipMath=false;
	ShowMoreMath=false;
	ShowHealingBar=true;
	TargetGlue=false;          -- if true, updates/shows bar as soon as target changes
	ShowButtonText=true;       -- displays average heal on spell buttons
	Spellbonus={};             -- table of arbitrary spell bonuses
	};
HealingEstimatorCharData={};	-- Spell data goes here

-- hook tooltip code
GameTooltip.SetAction_HealEstOld = GameTooltip.SetAction;

function GameTooltip:SetAction(slot)

    GameTooltip:SetAction_HealEstOld(slot);
    
    -- check if we should show anything
    if (not HealingEstimatorData.ShowMoreMath and not HealingEstimatorData.ShowSpellTipMath) then return; end
	
	-- fancy stuff follows
	local ST_left_text = GameTooltipTextLeft1:GetText();
	if (ST_left_text ~= nil) then
		local ST_text, ST_name, ST_rank, ST_cooldown, ST_tools, ST_reagents, ST_instant, i;
		local ST_right_text = GameTooltipTextRight1:GetText();
		
		local theSpellName = ST_left_text;
		local theSpellRank = ST_right_text;
		
		--[[ check class
        if ( HealingEstimator.PlayerClass == HE_CL_SHAMAN ) then
            -- no special code for this class
            -- so just fetch empirical healing estimate
            -- and add to tooltip
            if (HealingEstimatorCharData[theSpellName]) then
                if (HealingEstimatorCharData[theSpellName][theSpellRank]) then
                    local SpellHealAmount = HealingEstimatorCharData[theSpellName][theSpellRank];
                    GameTooltip:AddLine(" ",0,1,0);
                    GameTooltip:AddDoubleLine("Empirically estimated avg heal",HealingEstimator_Round(SpellHealAmount) );
                end
            end
            -- update tooltip
            GameTooltip:Show();
            return;
        end ]]--

		ST_right_text = ST_right_text or "";
		
		local isHOT = false;
		if ( theSpellName==HE_SPELL_Renew or theSpellName=="Lightwell" or theSpellName==HE_SPELL_Rejuvenation ) then isHOT = true; end
		
        if ( HealingEstimator_IsHealingSpell(theSpellName) or isHOT ) then
           -- is healing spell, so additional info
           
           -- compute average non-crit heal
           local avgnoncrit;
           local altnoncrit = 0;
           local duration=0;
           local altduration = 0;
            avgnoncrit=1;
            local Text=GameTooltipTextLeft4:GetText();
            local xxv = string.sub(Text, 1, 8);
            if xxv=="Cooldown" then
                Text = GameTooltipTextLeft5:GetText();
            end
            if (isHOT) then
                -- HOT
                if (Text) then
                    if theSpellName=="Lightwell" then
                        for t1,t2 in string.gmatch(Text, "(%d+) health over (%d+)") do
                            if (t2 == nil) then t2 = t1; end
                            avgnoncrit=t1;
                            duration=t2;
                            break;
                        end
                    else
                        for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.HOTValue) do
                            if (t2 == nil) then t2 = t1; end
                            avgnoncrit=t1;
                            duration=t2;
                            break;
                        end
                    end
                end
            else   
                -- not a HOT
                if ( theSpellName == HE_SPELL_PrayerOfMending ) then
                    if (Text) then
                        -- need special text capture for PoM
                        for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.PoMHealValue) do
                            --HealingEstimator_Print(true,"PoM Heal: "..t1);
                            avgnoncrit=t1;
                            break;
                        end
                    end
                elseif ( theSpellName == HE_SPELL_PowerWordShield ) then
                    if (Text) then
                        -- need special text capture for PoM
                        for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.PWSAbsorb) do
                            --HealingEstimator_Print(true,"PoM Heal: "..t1);
                            avgnoncrit=t1;
                            break;
                        end
                    end
                elseif (Text) then
                    for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.HealValue) do
                        --HealingEstimator_Print(true,"Heal: "..t1.." - "..t2);
                        if (t2 == nil) then t2 = t1; end
                        avgnoncrit=(t1+t2)/2;
                        break;
                    end
                end
                -- check if has secondary HOT effect
                if ( theSpellName==HE_SPELL_Regrowth ) then
                    if (Text) then
                        for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.HOTValue) do
                            --HealingEstimator_Print(true,"HOT: "..t1.." - "..t2);
                            if (t2 == nil) then t2 = t1; end
                            altnoncrit=t1;
                            altduration=t2;
                            break;
                        end
                    end
                end
            end
            
            if ( theSpellName==HE_SPELL_Lifebloom ) then
                avgnoncrit = 600;
                altnoncrit = 273;
            end
           
           local BoLtarget = "target";
           if ( not UnitExists("target") ) then
                BoLtarget = "player";
           end
           local bolxtrahealing = HealingEstimator_GetBoLBonusHealing( theSpellName , BoLtarget );
           local thebonushealing = GetSpellBonusHealing(); -- + bolxtrahealing;
           avgnoncrit = avgnoncrit + bolxtrahealing;
           
           local predictedheal, castlevel, effcoeff = HealingEstimator_CalcAvgBonusHeal( theSpellName , theSpellRank , avgnoncrit , false , thebonushealing );
           
           local predictedheal2, castlevel2, effcoeff2;
           predictedheal2 = 0;
           if ( (altnoncrit*1) > 0 ) then
                -- compute secondary HOT effect
                predictedheal2, castlevel2, effcoeff2 = HealingEstimator_CalcAvgBonusHeal( theSpellName , theSpellRank , altnoncrit , true , thebonushealing );
           end
                      
           -- get mana cost
           local manacost = GameTooltipTextLeft2:GetText();
           local baseregen, castingregen = GetManaRegen();
            baseregen = floor( (baseregen * 5.0) + 0.5);
            castingregen = floor( (castingregen * 5.0) + 0.5);
           manacostlen = strlen(manacost);
           manacost = strsub(manacost , 1 , manacostlen-4 );
           
           -- find casting time
           local casttime = 1.5;
           local casttext = GameTooltipTextLeft3:GetText();
           if (casttext ~= HealingEstimatorLoc.Instant) then
                for t1 in string.gmatch(casttext, HealingEstimatorLoc.Casttime) do
                    casttime=t1;
                    break;
                end
            else
                -- is an instant cast
                casttime = 1.5;   -- global cooldown
            end
           
           -- check for druid feral absent mana cost problem
           if ( tonumber(manacost) == nil ) then
                --HealingEstimator_Print(true, "NIL!" );
                return;
            end
           if ( tonumber(manacost) ~= (manacost + 0) ) then return; end
           -- compute true mana cost
           --if ( theSpellName=="Flash Heal") then casttime=1.5; end
           local truemanacost = manacost - casttime * castingregen/5;
           if isHOT then truemanacost = manacost - duration * castingregen/5; end
           
           -- compute hpm
           local hpm, truehpm, hpm2, hpm3, hpm5, thpm2, thpm3, thpm5;
            if ( theSpellName == HE_SPELL_BindingHeal ) then
                -- double amount healed for Binding Heal
                predictedheal = predictedheal*2;
           end
           hpm = predictedheal/manacost;
           truehpm = predictedheal/truemanacost;
           if ( theSpellName == HE_SPELL_PrayerOfMending ) then
                -- compute efficiencies for completed jumps
                hpm2 = predictedheal*2/manacost;
                hpm5 = predictedheal*5/manacost;
                thpm2 = predictedheal*2/truemanacost;
                thpm5 = predictedheal*5/truemanacost;
           end
           if ( theSpellName == HE_SPELL_PrayerOfHealing ) then
                -- compute efficiencies for number of players healed
                hpm3 = predictedheal*3/manacost;
                hpm5 = predictedheal*5/manacost;
                thpm3 = predictedheal*3/truemanacost;
                thpm5 = predictedheal*5/truemanacost;
           end
           if ( theSpellName == "Circle of Healing" ) then
                -- compute efficiencies for number of players healed
                hpm3 = predictedheal*3/manacost;
                hpm5 = predictedheal*5/manacost;
                thpm3 = predictedheal*3/truemanacost;
                thpm5 = predictedheal*5/truemanacost;
           end
           if ( theSpellName == HE_SPELL_ChainHeal ) then
                -- compute efficiencies for completed jumps
                hpm2 = predictedheal*(1.5)/manacost;
                hpm3 = predictedheal*(1.75)/manacost;
                thpm2 = predictedheal*(1.5)/truemanacost;
                thpm3 = predictedheal*(1.75)/truemanacost;
           end
          
           
           -- compute hps
           if ( duration == 0 ) then duration=casttime; end
           local hps = predictedheal/duration;
           
           -- compute time-to-empty
           --   this is how many seconds spell can be cast until mana is gone
           --   computes using CURRENT mana, not max mana
           local plMana = UnitMana("player");
           local timetoempty = 1;
           if ( not isHOT ) then
                timetoempty = plMana/truemanacost * casttime;
            else
                timetoempty = plMana/(manacost - castingregen/5*duration) * duration;
            end
           
           -- compute total-healing remaining
           --   this is how much health can still be pumped out,
           --   based upon time-to-empty
           local totalhremaining = plMana/truemanacost * predictedheal;
           if ( isHOT ) then
                totalhremaining = plMana/(manacost - castingregen/5*duration) * predictedheal;
           end
           
           -- format endurance measures
           if ( timetoempty > 0 ) then
                -- positive, so round
                totalhremaining = format("%.0f",totalhremaining);
                if ( timetoempty < 60 ) then
                    timetoempty = format("%.0f",timetoempty).." sec";
                else
                    timetoempty = format("%.2f",timetoempty/60).." min";
                end
           else
                timetoempty = "Forever";
                totalhremaining = "Infinite";
           end
           
           if (HealingEstimatorData.ShowMoreMath or HealingEstimatorData.ShowSpellTipMath) then GameTooltip:AddLine(" "); end
           
           if (HealingEstimatorData.ShowMoreMath) then
                -- full info on many lines
               GameTooltip:AddDoubleLine( "Cast level",castlevel);
               GameTooltip:AddDoubleLine( "Bonus healing coefficient", format("%.2f",effcoeff));
               if ( predictedheal-avgnoncrit > 0 ) then
                   GameTooltip:AddDoubleLine( "Avg non-crit heal", HealingEstimator_Round(avgnoncrit).." + "..HealingEstimator_Round(predictedheal-avgnoncrit) , NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b , 0,1,0);
                else
                    GameTooltip:AddDoubleLine( "Avg non-crit heal", HealingEstimator_Round(avgnoncrit) , NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b , 0,1,0);
                end
               --GameTooltip:AddDoubleLine( "Mana cost", tonumber(manacost) );
               GameTooltip:AddDoubleLine( "True mana cost", truemanacost );
               GameTooltip:AddDoubleLine( "(True) Health-per-mana", "("..format("%.2f",truehpm)..") "..format("%.2f",hpm) );
               --GameTooltip:AddDoubleLine( "True health-per-mana", format("%.2f",truehpm) );
               
               -- PoM
               if ( theSpellName == HE_SPELL_PrayerOfMending ) then
                    GameTooltip:AddDoubleLine( "    Two targets", "("..format("%.2f",thpm2)..") "..format("%.2f",hpm2) );
                    GameTooltip:AddDoubleLine( "    Five targets", "("..format("%.2f",thpm5)..") "..format("%.2f",hpm5) );
               end
               
               -- PoH
               if ( theSpellName == HE_SPELL_PrayerOfHealing or theSpellName == "Circle of Healing" ) then
                    GameTooltip:AddDoubleLine( "    Three targets", "("..format("%.2f",thpm3)..") "..format("%.2f",hpm3) );
                    GameTooltip:AddDoubleLine( "    Five targets", "("..format("%.2f",thpm5)..") "..format("%.2f",hpm5) );
               end
               
               -- Shaman Chain Heal
               if ( theSpellName == HE_SPELL_ChainHeal ) then
                    GameTooltip:AddDoubleLine( "    Two targets", "("..format("%.2f",thpm2)..") "..format("%.2f",hpm2) );
                    GameTooltip:AddDoubleLine( "    Three targets", "("..format("%.2f",thpm3)..") "..format("%.2f",hpm3) );
               end
               
               GameTooltip:AddDoubleLine( "Health-per-second", format("%.2f",hps) );
               
               if ( isHOT ) then
                    if ( theSpellName=="Lightwell" ) then
                        GameTooltip:AddDoubleLine( "Health-per-tick", HealingEstimator_Round( predictedheal/(duration/2) ) );
                    else
                        GameTooltip:AddDoubleLine( "Health-per-tick", HealingEstimator_Round( predictedheal/(duration/3) ) );
                    end
               end
               
               -- endurance measures
               GameTooltip:AddDoubleLine( "Time to empty", timetoempty );
               --GameTooltip:AddDoubleLine( "Health remaining", totalhremaining );
               
               -- finally empirical average
               if (HealingEstimatorCharData[theSpellName]) then
                    if (HealingEstimatorCharData[theSpellName][theSpellRank]) then
                        local SpellHealAmount = HealingEstimatorCharData[theSpellName][theSpellRank];
                        GameTooltip:AddDoubleLine("Empirically estimated avg heal",HealingEstimator_Round(SpellHealAmount) );
                    end
                end
            end
           
           if (HealingEstimatorData.ShowSpellTipMath) then
                -- minimal info on one line
                local theline = "x";
                if ( isHOT ) then
                    theline = "Avg "..HealingEstimator_Round(predictedheal).." • "..format("%.2f",hpm).."hpm • "..HealingEstimator_Round( predictedheal/(duration/3) ).."hpt";
                else
                    theline = "Avg "..HealingEstimator_Round(predictedheal).." • "..format("%.2f",hpm).."hpm • "..format("%.2f",hps).."hps";
                end
                GameTooltip:AddLine(theline,0,1,0);
                if ( predictedheal2 > 0 ) then
                    theline = "Avg "..HealingEstimator_Round(predictedheal2).." • "..format("%.2f",predictedheal2/manacost).."hpm • "..HealingEstimator_Round( predictedheal2/(altduration/3) ).."hpt";
                    GameTooltip:AddLine("Secondary effect");
                    GameTooltip:AddLine(theline,0,1,0);
                end
                
                -- add avg heal to actionbutton itself
                --[[if ( HealingEstimatorData.ShowButtonText ) then
                    local aButton = getglobal("ActionButton"..slot.."Count");
                    if ( aButton ) then
                        aButton:SetText(HealingEstimator_Round(predictedheal));
                    end
                end]]--
                
           end
           
            if (HealingEstimatorData.ShowMoreMath or HealingEstimatorData.ShowSpellTipMath) then
                -- blessing of light notice
                if ( bolxtrahealing > 0 ) then
                    GameTooltip:AddLine("Calculations assume Blessing of Light",0.5,0.5,0.5);
                end
                -- arbitrary bonus notice
                if HealingEstimatorData.Spellbonus then
                    if ( HealingEstimatorData.Spellbonus[theSpellName] ) then
                        local bh = HealingEstimatorData.Spellbonus[theSpellName];
                        GameTooltip:AddLine("Calculations use arbitrary bonus of "..bh.." +healing",0.5,0.5,0.5);
                    end
                end
            end
           
           -- make healing meter show amount of currently inspected heal spell
            SpellHealAmount = predictedheal;
            HealingEstimatorHealingBar:SetValue(SpellHealAmount);
            UpdateHealth=true;

        end
        -- update tooltip
        GameTooltip:Show();
        
    end
end

function HealingEstimator_Round(x)

	if ( (x+0) > 0) then
		return math.floor(x+0.5);
	else
		return math.ceil(x-0.5);
	end
end
--------------------------------------------------------------------
-- Debug print
--
function HealingEstimator_Print(Visible, Txt)
	if (Visible and Txt) then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFA0A0FF[HealingEstimator]|r "..Txt,0.5,0.5,1);
	end
end

--------------------------------------------------------------------
-- Initialization
--------------------------------------------------------------------

--------------------------------------------------------------------
-- OnLoad
--
function HealingEstimator_OnLoad()
	-- Register events

	-- For initialization
	this:RegisterEvent("VARIABLES_LOADED");
end

--------------------------------------------------------------------
-- Init
--
function HealingEstimator_Init()
	if (Loaded) then
		return;
	end
	Loaded=true;
    
	-- Check if addon needs to be enabled
	if ( not HealingEstimator_IsHealingClass(UnitClass("player")) ) then
		return;
	end
	
	-- toy event catching
	this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	
	-- catch equipment changes so we can update chart, if shown
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");

	-- When player's target changes
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	-- When unit health changes
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("UNIT_MAXHEALTH");
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	-- Healing spell you casted
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS");
	-- new events for buff detection, for HOTs and Fear Ward
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS");
	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER");
	this:RegisterEvent("CHAT_MSG_SPELL_PARTY_BUFF");
	this:RegisterEvent("UNIT_AURA");
	-- For spells
	this:RegisterEvent("UNIT_SPELLCAST_SENT");
	--this:RegisterEvent("UNIT_SPELLCAST_START");
	this:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	this:RegisterEvent("UNIT_SPELLCAST_STOP");
	this:RegisterEvent("UNIT_SPELLCAST_FAILED");
	this:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");

	-- Set slash command
	SlashCmdList["HEALINGESTIMATORCOMMAND"] = HealingEstimator_SlashHandler;
	SLASH_HEALINGESTIMATORCOMMAND1 = "/healingestimator";
	SLASH_HEALINGESTIMATORCOMMAND2 = "/healest";

	-- Init data
	GroupMaxHeal={};
	GroupMaxHeal[UnitName("player")]=UnitHealthMax("player")-UnitHealth("player");

	-- Set scale
	HealingEstimator:SetScale( HealingEstimatorData.Scale );

	-- Set BarBG color
	HealingEstimatorBarBG:SetVertexColor(0.0,0.0,0.0, 0.8);

	-- Initialize the spell target and rank resolving engine
	--HealingEstimator_InitTargetEngine();
	
	-- Replace text parsing functions on "difficult" locales
	local Locale=GetLocale();
	if (Locale=="frFR") then
		HealingEstimator_ParseSelfBuff = HealingEstimator_ParseSelfBuff_FR;
        HealingEstimator_ParsePeriodicSelfBuff = HealingEstimator_ParsePeriodicSelfBuff_FR;
	end

	-- Print welcome message
	HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome);
	
end

--------------------------------------------------------------------
-- Slash command handling
--------------------------------------------------------------------
function HealingEstimator_mp5stats( mp5value )
    -- converts mana per 5 to mana per minute
    HealingEstimator_Print( true , mp5value.." mana/5sec = "..mp5value * (60/5) .." mana/min." );
end

function HealingEstimator_spiritstats( spi )
    -- converts spirit to equivalent mana/5 values
    pertick = ( (spi / 4) * 10 ) / 10;
    perfive = pertick * 5 / 2;
    
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive ).." mana/5sec ("..HealingEstimator_Round( perfive*60/5 ).."/min) at 100% regen." );
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive*0.15 ).." mana/5sec ("..HealingEstimator_Round( 0.15*perfive*60/5 ).."/min) at 15% regen." );
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive*0.2 ).." mana/5sec ("..HealingEstimator_Round( 0.2*perfive*60/5 ).."/min) at 20% regen." );
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive*0.3 ).." mana/5sec ("..HealingEstimator_Round( 0.3*perfive*60/5 ).."/min) at 30% regen." );
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive*0.35 ).." mana/5sec ("..HealingEstimator_Round( 0.35*perfive*60/5 ).."/min) at 35% regen." );
    HealingEstimator_Print( true , spi.." Spirit = "..HealingEstimator_Round( perfive*5 ).." mana/5sec ("..HealingEstimator_Round( 5*perfive*60/5 ).."/min) at 500% regen (Innervate). That's equivalent to "..HealingEstimator_Round( 5*perfive/2*9 ).." to "..HealingEstimator_Round( 5*perfive/2*10 ).." over the whole course of one Innervate." );
end
--------------------------------------------------------------------
-- Slash handler
--
function HealingEstimator_SlashHandler(msg)
    
    -- split off any parameters
    local param;
    msg,param = strsplit(" ",msg,2);
    
	if (msg=="debug") then
		if (DebugMsg) then
			DebugMsg=true;
		else
			DebugMsg=true;
		end
    elseif (msg=="mp5") then
        HealingEstimator_mp5stats( param );
    elseif (msg=="spi2mp5") then
        HealingEstimator_spiritstats( param );
    elseif (msg=="spellbonus") then
        -- adds a fixed healing bonus to a named spell
        -- split up the params
        -- format should be Spell Name-bonus
        local sn,sb = strsplit("-",param,2);
        --HealingEstimator_Print(true,sn.."/"..sb);
        
        -- check if table exists
        if ( not HealingEstimatorData.Spellbonus ) then
            HealingEstimatorData.Spellbonus = {};
        end
        
        -- add new bonus
        HealingEstimatorData.Spellbonus[sn] = sb;
        
        -- clear bonus
        if ( tonumber(sb)==0 and HealingEstimatorData.Spellbonus ) then
            if ( HealingEstimatorData.Spellbonus[sn] ) then
                -- remove a bonus from the table
                HealingEstimatorData.Spellbonus[sn] = nil;
            end
        end
    elseif (msg=="buttontext") then
        if ( HealingEstimatorData.ShowButtonText ) then
            HealingEstimatorData.ShowButtonText = false;
            HealingEstimator_Print(true,"No longer displaying average heal on spell buttons.");
        else
            HealingEstimatorData.ShowButtonText = true;
            HealingEstimator_Print(true,"Now displaying average heal on spell buttons.");
        end
    elseif (msg=="visible") then
        if ( HealingEstimatorData.ShowHealingBar ) then
            HealingEstimatorData.ShowHealingBar = false;
        else
            HealingEstimatorData.ShowHealingBar = true;
        end
    elseif ( msg == "glue" ) then
        HealingEstimatorData.TargetGlue = (not HealingEstimatorData.TargetGlue);
        if HealingEstimatorData.TargetGlue then
            HealingEstimator_Print(true,"Bar will now stick to target.");
        else
            HealingEstimator_Print(true,"Bar will no longer stick to target.");
        end
    elseif (msg=="chart") then
        if ( not HealingEstimatorChartFrame:IsShown() ) then
            HealingEstimator_showChart();
        end
        HealingEstimator_doChart();
    elseif (msg=="portrait") then
        if ( HealingEstimatorData.ShowPortrait ) then
            HealingEstimatorPortrait:Hide();
            HealingEstimatorCircleFrame:Hide();
            HealingEstimatorData.ShowPortrait = false;
        else
            HealingEstimatorPortrait:Show();
            HealingEstimatorCircleFrame:Show();
            HealingEstimatorData.ShowPortrait = true;
        end
        HealingEstimatorMenuShowPortrait:SetChecked( HealingEstimatorData.ShowPortrait );
    elseif (msg=="math") then
        if ( HealingEstimatorData.ShowSpellTipMath ) then
            HealingEstimatorData.ShowSpellTipMath = false;
        else
            HealingEstimatorData.ShowSpellTipMath = true;
            HealingEstimator_Print(InfoMsg,"Heal math shown. hpm = health-per-mana. hps = health-per-second. hpt = health-per-tick.");
        end
    elseif (msg=="moremath") then
        if ( HealingEstimatorData.ShowMoreMath ) then
            HealingEstimatorData.ShowMoreMath = false;
        else
            HealingEstimatorData.ShowMoreMath = true;
        end
	else
		-- Show menu and help text
		HealingEstimatorMenu:Show();
		HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome2);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome3);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome4);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome5);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome6);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome7);
        HealingEstimator_Print(InfoMsg,HealingEstimatorLoc.Welcome8);
	end
end

--------------------------------------------------------------------
-- OnEvent
--
function HealingEstimator_OnEvent(event)
	HealingEstimator_Print(DebugMsg,event);
	
	if (event == "UPDATE_MOUSEOVER_UNIT") then
		if (UnitName("mouseover")) then
			HealingEstimator_UnitFormat("mouseover");
		end
    end
    
    if (event == "UNIT_INVENTORY_CHANGED" and HealingEstimatorChartFrame:IsShown()) then
        -- chart is open, so update
        HealingEstimator_doChart();
    end
	
	if (event=="UNIT_SPELLCAST_SENT") then
	   --HealingEstimator_Print(true,arg1..arg2..arg3..arg4);
	   HealingEstimator_StartSpell(arg2,arg3,arg4);
	end
	
	if ( event=="PLAYER_TARGET_CHANGED" and HealingEstimatorData.TargetGlue ) then
	   -- trigger a spell start, but flag it as bar display event
	   HealingEstimator_ShowBar(UnitName("target"));
	end

	if (event=="UNIT_HEALTH" or event=="UNIT_MAXHEALTH") then
		-- Unit health changed
		-- Update max heal
		local Name=UnitName(arg1);
		if (Name) then
			GroupMaxHeal[Name]=UnitHealthMax(arg1)-UnitHealth(arg1);
			HealingEstimator_Print(DebugMsg,Name..": "..GroupMaxHeal[Name]);
		end
		
		-- Check if this is spell target
		HealingEstimator_Print(DebugMsg,SpellTarget);
		if (arg1==SpellTarget) then
			UpdateHealth=true;
			
			-- Update maximum values if needed
			if (event=="UNIT_MAXHEALTH") then
				local MaxHealth=UnitHealthMax(arg1);
				HealingEstimatorBar:SetMinMaxValues(0,MaxHealth);
				HealingEstimatorHealingBar:SetMinMaxValues(0,MaxHealth);
				HealingEstimatorHealFlash:SetWidth( (SpellHealAmount*barwidth)/MaxHealth );
			end
		end
    elseif ( event=="CHAT_MSG_SPELL_AURA_GONE_SELF" or event=="CHAT_MSG_SPELL_AURA_GONE_OTHER" or event=="CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS" or event=="CHAT_MSG_SPELL_PARTY_BUFF" or event=="CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" or event=="UNIT_AURA") then
        -- buff fades, just need to register bar for update
        UpdateHealth=true;
	elseif (event=="PARTY_MEMBERS_CHANGED") then
		-- Party changed
		-- Clear data
		GroupMaxHeal={};
		-- Set player data
		GroupMaxHeal[UnitName("player")]=UnitHealthMax("player")-UnitHealth("player");

		-- Set group data
		local c;
		local Group,Count=HealingEstimator_GetGroupInfo();
		for c=1,Count do
			local Name=Group..c;
			if (UnitName(Name)) then
				GroupMaxHeal[UnitName(Name)]=UnitHealthMax(Name)-UnitHealth(Name);
			end
		end

	elseif (event=="UNIT_SPELLCAST_START") then
		-- Spellcast started, get spell info
		--SpellTime=arg2/1000;
		--HealingEstimator_StartSpell(arg1);
	elseif (event=="UNIT_SPELLCAST_DELAYED") then
		-- Spellcast was delayed, add cast time
		--SpellTime=SpellTime+(arg1/1000);
	elseif (event=="UNIT_SPELLCAST_FAILED") then
		-- Spell failed
		SpellName=nil;
		OverhealDetected=false;
	elseif (event=="UNIT_SPELLCAST_INTERRUPTED") then
		-- Spell interrupted
		SpellName=nil;
		OverhealDetected=false;
	elseif (event=="UNIT_SPELLCAST_STOP") then
		-- Spell finished
		SpellName=nil;
		OverhealDetected=false;

	elseif (event=="CHAT_MSG_SPELL_SELF_BUFF") then
		-- You healed someone
		-- Parse combat log message at localized function
		HealingEstimator_AddHealing( HealingEstimator_ParseSelfBuff(arg1) );
		
		-- Show flash
		HealingEstimatorFlash:Show();
		HealingEstimatorHealFlash:SetPoint("LEFT","HealingEstimatorHealingBar","LEFT");
		HealingEstimatorHealFlash:Show();

	elseif (event=="CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" or
			event=="CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS" or
			event=="CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS" or
			event=="CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS" ) then
		-- Your heal-over-time healed someone
		-- Parse combat log message at localized function
		HealingEstimator_AddHealing( HealingEstimator_ParsePeriodicSelfBuff(arg1) );

	elseif (event == "VARIABLES_LOADED") then
		-- Variables loaded, do initialization
		HealingEstimator_Init();
	end
end

--------------------------------------------------------------------
-- Misc functions
--------------------------------------------------------------------

--------------------------------------------------------------------
-- Adds healing data
--
function HealingEstimator_AddHealing(Spell, Target, Heal, Crit)
	-- If spell name and heal value was resolved
	if (not (Spell and Target and Heal)) then
		-- Was not, bail out
		return;
	end

	-- Update healing amount
	Heal=Heal+0;		-- Convert healing amount to number
	TotalHealing=TotalHealing+Heal;
	
	-- check if was critical heal
	if ( Crit ) then
	   TotalCritHealing = TotalCritHealing + Heal;
	end
	
	-- Check that unit data exists
	if (not GroupMaxHeal[Target]) then 
		-- Somehow unit data was missing, do not process further
		return;
	end
	local MaxHeal=GroupMaxHeal[Target];

	-- Check overhealing
	if (Heal>MaxHeal) then
		TotalOverHealing=TotalOverHealing+(Heal-MaxHeal);
		local Over=floor(((Heal-MaxHeal)/Heal)*100);
		HealingEstimator_Print(DebugMsg,Over.."% overheal");
		-- check if was critical heal
        if ( Crit ) then
           TotalCritOverHealing = TotalCritOverHealing + (Heal-MaxHeal);
        end
	end
	HealingEstimatorPie_SetValue(TotalOverHealing/TotalHealing);

	-- Make sure that data exists
	HealingEstimator_Print(DebugMsg,Spell.." "..Heal);
	if (not HealingEstimatorCharData[Spell]) then return; end
	if (not HealingEstimatorCharData[Spell][SpellRank]) then return; end
	local Avg=HealingEstimatorCharData[Spell][SpellRank];
	if (Avg==0) then
		Avg=Heal;
	else
		Avg=(Avg+Heal)/2;
	end
	if ( not Crit ) then HealingEstimatorCharData[Spell][SpellRank]=Avg; end
end

--------------------------------------------------------------------
-- Returns healing data
--
function HealingEstimator_GetData()
	--return TotalHealing,TotalOverHealing;
	return TotalHealing,TotalOverHealing,TotalCritHealing,TotalCritOverHealing;
end

--------------------------------------------------------------------
-- Clears healing data
--
function HealingEstimator_ClearData()
	TotalHealing=0;
	TotalOverHealing=0;
	TotalCritHealing=0;
	TotalCritOverHealing=0;
	HealingEstimatorPie_SetValue(0);
	HealingEstimator_Print(InfoMsg,"Healing data cleared");
end

--------------------------------------------------------------------
-- Forces bar visible
--
function HealingEstimator_ForceVisible(Value)
	ForceVisible=Value;
	if (ForceVisible) then
		HealingEstimatorText:SetText(HealingEstimatorLoc.TargetName);
		HealingEstimatorFlash:Hide();
		HealingEstimatorHealFlash:Hide();
		HealingEstimator:SetAlpha(1);
		HealingEstimator:Show();
	else
		HealingEstimator:Hide();
	end	
end

--------------------------------------------------------------------
-- Tells if heal bar can be moved
--
function HealingEstimator_CanMove()
	if (IsShiftKeyDown() or ForceVisible) then
		return true;
	end
	return false;
end

--------------------------------------------------------------------
-- Gets the "name" and count of group
-- i.e. returns "party" or "raid", so those can be used like "partyX" or "raidX"
--
function HealingEstimator_GetGroupInfo()
	-- Assume that player is in raid
	local Group="raid";
	local Count=GetNumRaidMembers();

	-- If he is not in the group, assume party
	if (Count==0) then
		Group="party";
		Count=GetNumPartyMembers();
	end

	-- If he isn't even in raid, then he is alone
	if (Count==0) then
		Group=nil;
		Count=0;
	end

	-- Return group data
	return Group,Count;
end

--------------------------------------------------------------------

function HealingEstimator_GetGroupName(name)
  if name == UnitName("player") then
    return "player";
  end
  local num = GetNumRaidMembers();
  local i;
  for i = 1, num do
      local n = UnitName("raid"..i);
      if n == name then
          return "raid"..i;
      end
  end
  num = GetNumPartyMembers();
  for i = 1, num do
      local n = UnitName("party"..i);
      if n == name then
          return "party"..i;
      end
  end
  return nil;
end

--------------------------------------------------------------------
-- Sets visual data of the unit
--
function HealingEstimator_SetTargetData(Unit,HealAverage)
	-- Check that unit is not nil
	if (not Unit) then
		return;
	end
    
	-- Store target name
	SpellTargetName=UnitName(Unit);
	
	--HealingEstimator_Print(true,Unit);

	-- Set target health bar
	local Health=UnitHealth(Unit);
	local MaxHealth=UnitHealthMax(Unit);
	local HealthDeficit=MaxHealth-Health;
	HealingEstimatorBar:SetMinMaxValues(0,MaxHealth);
	HealingEstimatorBar:SetValue(Health);
	if ( HealthDeficit==0 ) then
	   HealingEstimatorText:SetText(SpellTargetName);
	else
	   HealingEstimatorText:SetText(SpellTargetName .. " (-" .. HealthDeficit .. ")");
    end

	-- Set healing bar
	HealingEstimatorHealingBar:SetMinMaxValues(0,MaxHealth);
	HealingEstimatorHealingBar:SetValue(HealAverage);
	local Pos=(Health*barwidth)/MaxHealth;
	HealingEstimatorHealingBar:SetPoint("LEFT","HealingEstimatorBar","LEFT",Pos,0);
	HealingEstimatorHealingBar:SetStatusBarColor(1,1,0);

	-- Set flash bar width
	HealingEstimatorHealFlash:SetWidth( (HealAverage*barwidth)/MaxHealth );

	-- Show bars
	HealingEstimatorFlash:Hide();
	HealingEstimatorHealFlash:Hide();
	HealingEstimator:SetAlpha(1);
	HealingEstimator:Show();
	UpdateHealth=true;
end

-------

function HealingEstimator_StoreSpellTip(Name,Rank)

	-- Name is either "Flash Heal(Rank 1)" or "Flash Heal"
	-- Find the spell from spellbook
	local r,FoundRank=nil;
	--for r in string.gmatch(Name,HealingEstimatorLoc.RankTxt) do
	--	FoundRank=r;
		-- Remove the rank text
	--	Name=string.gsub(Name,HealingEstimatorLoc.RankTxt,"");
	--	break;
	--end

	-- Search the biggest rank
	local SpellId=1;
	local FoundSpell=nil;
	while true do
		-- Get spell name and rank
		local TempName,TempRank = GetSpellName(SpellId,BOOKTYPE_SPELL);
		if (not TempName) then 
			-- All spells tested, exit
			break;
		end
		
		-- Check spell name
		if (Name==TempName and Rank==TempRank) then
			FoundSpell=SpellId;
		end
		SpellId=SpellId+1;
	end
	
	-- If spell was found, get it's data
	HealingEstimatorTooltip:ClearLines();
	HealingEstimatorTooltip:SetOwner(HealingEstimator,"ANCHOR_NONE");
	if (FoundSpell) then
		_,TempRank=GetSpellName(FoundSpell,BOOKTYPE_SPELL);	-- Get rank
		HealingEstimatorTooltip:SetSpell(FoundSpell,BOOKTYPE_SPELL);
		HealingEstimatorTooltipTextRight1:SetText(TempRank);
	end
end


function HealingEstimator_FindMaxRank(aSpell)
    -- returns highest rank of aSpell in player's spellbook
    
	local SpellId=1;
	local MaxRank=0;
	local TempName,TempRank = GetSpellName(SpellId,BOOKTYPE_SPELL);
	while (TempName) do
    		
		-- Check spell name
		if (aSpell==TempName) then
            local ranklength = strlen(TempRank);
            TempRank = tonumber(strsub(TempRank , ranklength-1 , ranklength ));
			if ( TempRank > MaxRank ) then
                MaxRank = TempRank;
			end
		end
		SpellId=SpellId+1;
		
		-- Get spell name and rank
        TempName,TempRank = GetSpellName(SpellId,BOOKTYPE_SPELL);
	end
	
	return MaxRank;
	
end




function HealingEstimator_ShowBar(theTarget)
    -- pops up bar for new target
    SpellTarget=HealingEstimator_GetGroupName(theTarget);
    AcquiredTarget = SpellTarget;
    HealingEstimator_SetTargetData(SpellTarget , 0);
end



--------------------------------------------------------------------
-- StartSpell
-- Does the initialization for spellcast
--
function HealingEstimator_StartSpell(theSpell,theRank,theTarget )

    
	-- Check if spell is healing spell (localized function)
	if ( not HealingEstimator_IsHealingSpell(theSpell) or not HealingEstimatorData.ShowHealingBar ) then
		-- It wasn't, or we're set to never show the bar
		return;
	end
	
	-- get tooltip text
    HealingEstimator_StoreSpellTip(theSpell,theRank);

	-- Get spell info
	local HealAverage;
    --SpellName, SpellRank, HealAverage, SpellTarget = HealingEstimator_GetSpellInfo();
    -- Get spell healing average
	HealAverage=1;
	local Text=HealingEstimatorTooltipTextLeft4:GetText();
	if (Text) then
		for t1,t2 in string.gmatch(Text, HealingEstimatorLoc.HealValue) do
			--HealingEstimator_Print(true,"Heal: "..t1.." - "..t2);
			HealAverage=(t1+t2)/2;
			break;
		end
	end
    
    SpellName = theSpell;
    SpellRank = theRank;
    SpellTarget = theTarget;

    SpellTarget=HealingEstimator_GetGroupName(SpellTarget);
    AcquiredTarget = SpellTarget;
    
	-- Check that unit exists
	if (not SpellTarget) then
		-- There is no target, but still casting is going on
		-- Probably spell was run with selfcast, can't be sure
		return;
	end

	-- Make sure that spell data exists
	if (not HealingEstimatorCharData[SpellName]) then
		HealingEstimatorCharData[SpellName]={};
	end
	-- Check also that rank data exists
	if (not HealingEstimatorCharData[SpellName][SpellRank]) then
		-- The rank was not found, create it's data
		HealingEstimatorCharData[SpellName][SpellRank]={};
		HealingEstimatorCharData[SpellName][SpellRank]=HealAverage;
	end

	-- Get healing value from empirical data
	SpellHealAmount=HealingEstimatorCharData[SpellName][SpellRank];
	
	-- try to compute average heal
	if ( HealingEstimator_IsHealingSpell(SpellName) ) then
       -- is supported healing spell, so we try to calculate it with talents and such
       
       local bonushealing = GetSpellBonusHealing();
       
       HealAverage = HealAverage + HealingEstimator_GetBoLBonusHealing(SpellName,AcquiredTarget);
       
       -- finally do the math
       SpellHealAmount, _, _ = HealingEstimator_CalcAvgBonusHeal(SpellName,SpellRank,HealAverage, false, bonushealing);
       
    end
	
	HealingEstimator_SetTargetData(SpellTarget,SpellHealAmount);
end

function HealingEstimator_GetBoLBonusHealing(spell,unit)
    -- checks if blessing of light is on "unit"
    -- return appropriate bonus healing for spell
    
    local xtra = 0;
     -- check if paladin spell
   if ( spell==HE_SPELL_HolyLight or spell==HE_SPELL_FlashOfLight ) then
        -- check if target of spell has Blessing of Light
        local bolrank = HealingEstimator_CheckBoL(unit);
        if ( bolrank > 0 ) then
            if ( spell==HE_SPELL_HolyLight ) then
                xtra = HealingEstimator.BoL.holylight[bolrank];
            else
                xtra = HealingEstimator.BoL.flashoflight[bolrank];
            end
        end
   end
   return xtra;
end

------------

function HealingEstimator_GetHealTalentsByClass(aClass)
    -- returns a list of talent points
    
    local theTalents = {};
    
    if ( aClass==HE_CL_PRIEST ) then
        -- get empowered healing rank
        local _, _, _, _, currRankEH, _ = GetTalentInfo(2,20);
        -- get spiritual healing rank
        local _, _, _, _, currRankSH, _ = GetTalentInfo(2,16);
        -- get improved renew rank
        local _, _, _, _, currRankIR, _ = GetTalentInfo(2,2);
        theTalents = { EmpoweredHealing = currRankEH , SpiritualHealing = currRankSH , ImprovedRenew = currRankIR };
        
    end
    
    if ( aClass==HE_CL_DRUID ) then
        
        local _, _, _, _, currRankImRejuv, _ = GetTalentInfo(3,10); 
         
        local _, _, _, _, currRankGoN, _ = GetTalentInfo(3,12);
        
        local _, _, _, _, currRankET, _ = GetTalentInfo(3,14);
        
        local _, _, _, _, currRankER, _ = GetTalentInfo(3,19);
        
        theTalents = { ImprovedRejuvenation = currRankImRejuv , GiftOfNature = currRankGoN , EmpoweredTouch = currRankET , EmpoweredRejuvenation = currRankER };
    
    end
    
    if ( aClass == HE_CL_PALADIN ) then
    
        local _, _, _, _, currRankHealingLight, _ = GetTalentInfo(1,5);  
        
        theTalents = { HealingLight = currRankHealingLight };
        
    end
    
    if ( aClass == HE_CL_SHAMAN ) then
        
        -- improved 1, to all healing (5pts, 2% each)
        local _, _, _, _, currRankPurification, _ = GetTalentInfo(3,15);
        
        -- improved 2, just to chain heal (2pts, 10% each)
        local _, _, _, _, currRankImChainHeal, _ = GetTalentInfo(3,19);  
        
        theTalents = { Purification = currRankPurification , ImChainHeal = currRankImChainHeal };
        
    end
    
    return theTalents;
end

function HealingEstimator_GetHealAfterTalents( aSpellName , AvgHealAmount , basecoeff , castlevel , learnlevel , aClass , theTalents , aBonusHealing )
    -- compute effective coefficient
    local effbonuscoeff = 0;   
    effbonuscoeff = min( (castlevel + 6)/UnitLevel("player") , 1 );
    
    local tempowered1 = 0;
    local tempowered2 = 0;
    local timproved1 = 0;
    local timproved2 = 0;
    
    if ( aClass == HE_CL_PRIEST ) then
        -- extract talents
        tempowered1 = theTalents.EmpoweredHealing;
        timproved1 = theTalents.SpiritualHealing*0.02;
        timproved2 = theTalents.ImprovedRenew*0.05;
        
        if ( aSpellName == "Lightwell" ) then timproved1=0; end
        
        if ( aSpellName ~= HE_SPELL_Renew ) then timproved2=0; end
        if ( aSpellName == HE_SPELL_GreaterHeal ) then
           tempowered1 = tempowered1 * 0.04;
        elseif ( aSpellName == HE_SPELL_FlashHeal ) then
           tempowered1 = tempowered1 * 0.02;
        elseif ( aSpellName == HE_SPELL_BindingHeal ) then
           tempowered1 = tempowered1 * 0.02;
        else
           tempowered1 = 0;
        end
    end
    
    if ( aClass == HE_CL_DRUID ) then
    
        tempowered1 = theTalents.EmpoweredTouch * 0.1;
        if ( aSpellName ~= HE_SPELL_HealingTouch ) then tempowered1=0; end
        tempowered2 = theTalents.EmpoweredRejuvenation * 0.04;
        if ( aSpellName ~= HE_SPELL_Rejuvenation ) then tempowered2 = 0; end
        timproved1 = theTalents.GiftOfNature * 0.02;
        timproved2 = theTalents.ImprovedRejuvenation * 0.05;
        if ( aSpellName ~= HE_SPELL_Rejuvenation ) then timproved2=0; end  
        
    end
    
    if ( aClass == HE_CL_PALADIN ) then
    
        timproved1 = theTalents.HealingLight * 0.04;
    
    end
    
    if ( aClass == HE_CL_SHAMAN ) then
    
        timproved1 = theTalents.Purification * 0.02;
        
        if ( aSpellName == HE_SPELL_ChainHeal ) then
            timproved2 = theTalents.ImChainHeal * 0.1;
        end
    
    end
    
    -- account for spells learned before level 20
    if ( learnlevel < 20 and UnitLevel("player") > 39 ) then
        basecoeff = basecoeff - (20 - learnlevel)*0.0375 ;
    end
    
    effbonuscoeff = ( basecoeff + tempowered1 + tempowered2 ) * effbonuscoeff ;
    
    local appbonushealing = effbonuscoeff * aBonusHealing;
    
    -- compute average non-crit heal
    local avgnoncrit = AvgHealAmount;
    avgnoncrit = ( avgnoncrit + appbonushealing ) * (1 + timproved1) * (1 + timproved2);
    
    return avgnoncrit, effbonuscoeff;
end

function HealingEstimator_GetCastLevel(aSpellName,aSpellRank)
    local ranklength = strlen(aSpellRank);
    aSpellRank = tonumber(strsub(aSpellRank , ranklength-1 , ranklength ));
    
    local castlevel,basecoeff,altcoeff,learnlevel;
    learnlevel = 60;
    
    -- find spell in spell table
    local i,count;
    count = table.getn(HealingEstimator.Spells);
    for i=1,count do
        if ( HealingEstimator.Spells[i].spell == aSpellName ) then
            castlevel = HealingEstimator.Spells[i].castlevelbyrank[aSpellRank];
            basecoeff = HealingEstimator.Spells[i].basecoeff;
            altcoeff = HealingEstimator.Spells[i].altcoeff;
            if ( HealingEstimator.Spells[i].learnlevel ) then
                learnlevel = HealingEstimator.Spells[i].learnlevel[aSpellRank];
            end
            -- check for coeff table
            if ( HealingEstimator.Spells[i].basecoeffbyrank ) then
                basecoeff = HealingEstimator.Spells[i].basecoeffbyrank[aSpellRank];
            end
        end
    end
    
    return castlevel,basecoeff,altcoeff,learnlevel;
end


function HealingEstimator_CalcAvgBonusHeal(aSpellName,aSpellRank,AvgHealAmount,altflag,totalbonushealing)

    -- this function applies math to plain average heal amount
    --   to get average non-crit heal after all healing bonuses
    -- works for priest right now, only

    local isHOT = false;
    local effectduration = 1;
    
    --if ( aSpellName == HE_SPELL_Renew or aSpellName == HE_SPELL_Rejuvenation ) then isHOT=true; end

    local castlevel = 60;
    local learnlevel = 60;
    local casttime = 3;
    local basecoeff = casttime/3.5;
    local altcoeff = 0; -- for spells with separate effects
    
    castlevel, basecoeff, altcoeff, learnlevel = HealingEstimator_GetCastLevel(aSpellName,aSpellRank);
    
    if ( altflag ) then
        basecoeff = altcoeff;
        if ( aSpellName == HE_SPELL_Regrowth ) then
            isHOT = true;
            casttime = 1;
            effectduration = 12;
        end 
    end
    
    local currTalents = HealingEstimator_GetHealTalentsByClass(HealingEstimator.PlayerClass);
    
    -- arbitrary bonus detection
    if HealingEstimatorData.Spellbonus then
        if ( HealingEstimatorData.Spellbonus[aSpellName] ) then
            totalbonushealing = totalbonushealing + HealingEstimatorData.Spellbonus[aSpellName];
            --HealingEstimator_Print( true , aSpellName.." / "..HealingEstimatorData.Spellbonus[aSpellName].." / "..totalbonushealing );
        end
    end
    
    local avgnoncrit,effbonuscoeff = HealingEstimator_GetHealAfterTalents( aSpellName , AvgHealAmount , basecoeff , castlevel, learnlevel , HealingEstimator.PlayerClass , currTalents , totalbonushealing );
    
    return avgnoncrit, castlevel, effbonuscoeff;

end

--------------------------------------------------------------------
-- OnUpdate
-- Updates the target health, healing & overhealing values
--
function HealingEstimator_OnUpdate(Time)
	-- Check that there is some spell
	--if (SpellName==nil or SpellTarget==nil) then
	
	--HealingEstimator_Print(true,UnitName("target"));
	
	if (SpellName==nil and (SpellTarget==nil or UnitName("target")==nil)) then
		--HealingEstimator:Hide();
		
		
		-- If this is forced to be visible don't fade out
		--ForceVisible=true;
		if (ForceVisible) then return; end
		
		-- Fade out
		local Alpha=this:GetAlpha()-Time;
		if (Alpha<0) then
			this:Hide();
		end
		-- Set alpha
		this:SetAlpha(Alpha);
		return;
	end
	
	-- fade out spell flash
    local fAlpha=HealingEstimatorFlash:GetAlpha()-(Time/2);
    if (fAlpha<0) then
        HealingEstimatorFlash:Hide();
        HealingEstimatorHealFlash:Hide();
    end
    -- Set flash alpha
    HealingEstimatorFlash:SetAlpha(fAlpha);
    HealingEstimatorHealFlash:SetAlpha(fAlpha);
	
	
	-- Update time
	SpellTime=SpellTime-Time;
    
    --UpdateHealth=true;
	-- Check if update is needed
	if (not UpdateHealth) then
		return;
	end
	UpdateHealth=false;

	-- Target is lost somehow
	if ( SpellTarget==nil ) then
	--if (UnitName(SpellTarget) ~= SpellTargetName) then
		SpellName=nil;
		OverhealDetected=false;
		return;
	end

	-- Find target health
	local Health=UnitHealth(SpellTarget);
	local MaxHealth=UnitHealthMax(SpellTarget);
	local MaxHeal=GroupMaxHeal[SpellTargetName];
	local HealthDeficit=MaxHealth-Health;

	-- Calculate healing and overhealing values
	local OverHealing=SpellHealAmount-MaxHeal;
	--HealingEstimator_Print(DebugMsg,Health.."/"..MaxHealth..": "..SpellHealAmount.."("..OverHealing..")");

	if (OverHealing < 0) then
		-- There is no overhealing
		OverHealing=0;
	end

	-- Update target health bar
	HealingEstimatorBar:SetValue(Health);
	if ( HealthDeficit==0 ) then
	   HealingEstimatorText:SetText(SpellTargetName);
	else
	   HealingEstimatorText:SetText(SpellTargetName .. " (-" .. HealthDeficit .. ")");
    end
    
    -- overheal percent to right of bar
    -- this shows percent over amount needed to heal to maximum
    if ( HealingEstimatorData.ShowOverhealPercent ) then    
        if ( OverHealing > 0 ) then
            HealingEstimatorText2:SetText( string.format( "%.0f" , ( (OverHealing/SpellHealAmount)*100 ) ).."%" );
        else
            HealingEstimatorText2:SetText( "-"..string.format( "%.0f" , ( ( (MaxHeal-SpellHealAmount)/MaxHeal )*100 ) ).."%" );
        end
    else
        -- show no text
        HealingEstimatorText2:SetText( "" );
    end
    

	-- Set color of healing bar
	-- Yellow (1,1,0) if overhealing is under the limit
	-- Red    (1,0,0) if there is overhealing
	local Green=1;
	local Ratio=OverHealing/SpellHealAmount;
	OverhealDetected=false;
	if (Ratio > HealingEstimatorData.Limit) then
		-- Overhealing detected, change color to red
		Green=0;
		OverhealDetected=true;
	end
	HealingEstimatorHealingBar:SetStatusBarColor(1,Green,0,0.75);

	-- Update healing bar position
	local Pos=(Health*barwidth)/MaxHealth;
	if (Pos>barwidth) then
		Pos=barwidth;
	end
	HealingEstimatorHealingBar:SetPoint("LEFT","HealingEstimatorBar","LEFT",Pos,0);
	
	
	-- set portrait
	SetPortraitTexture(HealingEstimatorPortrait, SpellTarget);
	
	
	-- finally update visibility of heal-over-time icons
	
	if ( HealingEstimatorData.ShowHealOverTimes ) then
	
        if ( HealingEstimator_HasHOT(SpellTarget,"rejuv") ) then
           HealingEstimatorRejuvIcon:Show();
           HealingEstimatorTextRejuvCount:Show();
        else
           HealingEstimatorRejuvIcon:Hide();
           HealingEstimatorTextRejuvCount:Hide();
        end
        
        if ( HealingEstimator_HasHOT(SpellTarget,"regrowth") ) then
           HealingEstimatorRegrowthIcon:Show();
           HealingEstimatorTextRegrowthCount:Show();
        else
           HealingEstimatorRegrowthIcon:Hide();
           HealingEstimatorTextRegrowthCount:Hide();
        end
        
        if ( HealingEstimator_HasHOT(SpellTarget,"renew") ) then
           HealingEstimatorRenewIcon:Show();
           HealingEstimatorTextRenewCount:Show();
        else
           HealingEstimatorRenewIcon:Hide();
           HealingEstimatorTextRenewCount:Hide();
        end
        
        if ( HealingEstimator_HasHOT(SpellTarget,"fearward") ) then
           HealingEstimatorFearWardIcon:Show();
        else
           HealingEstimatorFearWardIcon:Hide();
        end
        
        if ( HealingEstimator_HasHOT(SpellTarget,"bol") or HealingEstimator_HasHOT(SpellTarget,"gbol") ) then
           HealingEstimatorBoLIcon:Show();
        else
           HealingEstimatorBoLIcon:Hide();
        end
        
    else
        
        HealingEstimatorRenewIcon:Hide();
        HealingEstimatorTextRenewCount:Hide();
        HealingEstimatorRejuvIcon:Hide();
        HealingEstimatorTextRejuvCount:Hide();
        HealingEstimatorRegrowthIcon:Hide();
        HealingEstimatorTextRegrowthCount:Hide();
        HealingEstimatorFearWardIcon:Hide();
        HealingEstimatorBoLIcon:Hide();
	
	end
	
end


--CHECK FOR A HOT
--Returns true or false
function HealingEstimator_HasHOT(unit,spell)

    if ( spell == "rejuv" ) then
        --spell = "Interface\\Icons\\Spell_Nature_Rejuvenation";
        spell = HE_SPELL_Rejuvenation;
    elseif ( spell == "regrowth" ) then
        --spell = "Interface\\Icons\\Spell_Nature_ResistNature";
        spell = HE_SPELL_Regrowth;
    elseif ( spell == "renew" ) then
        --spell = "Interface\\Icons\\Spell_Holy_Renew";
        spell = HE_SPELL_Renew;
    elseif ( spell == "fearward" ) then
        --spell = "Interface\\Icons\\Spell_Holy_Excorcism";
        spell = HE_SPELL_FearWard;
    elseif ( spell == "bol" ) then
        spell = HE_SPELL_BlessingOfLight;
    elseif ( spell == "gbol" ) then
        spell = HE_SPELL_GrBlessingOfLight;
    else
        return false;
    end
    
    local renewcount = 0;
    local rejuvcount = 0;
    local regrowcount = 0;
    local buff_found = false;
	local i = 1;
	-- name, rank, icon, count = UnitBuff("unit", index [,raidFilter])
	local bname, brank, bicon, bcount = UnitBuff(unit, i);
    
	while bicon ~= nil do
	   --HealingEstimator_Print(true,bname..brank..bicon..bcount);
		if bname == spell then
            buff_found = true;
        end
        if bname == HE_SPELL_Renew then renewcount = renewcount + 1; end
        if bname == HE_SPELL_Rejuvenation then rejuvcount = rejuvcount + 1; end
        if bname == HE_SPELL_Regrowth then regrowcount = regrowcount + 1; end
		i = i + 1;
		bname, brank, bicon, bcount = UnitBuff(unit, i);
	end
	
    HealingEstimatorTextRenewCount:SetText(renewcount);
    HealingEstimatorTextRejuvCount:SetText(rejuvcount);
    HealingEstimatorTextRegrowthCount:SetText(regrowcount);
	
	--HealingEstimator_Print(true,"Found "..renewcount.." Renews");

	return buff_found;

end


function HealingEstimator_CheckBoL(unit)
    -- checks for Blessing of Light or Greater BoL on "unit"
    -- returns zero (0) if not present
    -- returns rank if present
    
    local i = 1;
    local bname, brank, bicon, bcount = UnitBuff(unit, i);
    local pat1 = HE_SPELL_BlessingOfLight;
    local pat2 = HE_SPELL_GrBlessingOfLight;
    
	while bicon ~= nil do
	    local ranklength = strlen(brank);
        brank = tonumber(strsub(brank , ranklength-1 , ranklength ));
        
		if ( bname == pat1 ) then
            return brank;
        end
        if ( bname == pat2 ) then
            return (brank + 2);   -- greater blessings are two ranks below normal ones
        end
		i = i + 1;
		bname, brank, bicon, bcount = UnitBuff(unit, i);
	end
	
	return 0;
end


-- cancel spell, if overhealing detected
function HealingEstimator_CancelSpell()
    SpellStopCasting();
end

function HealingEstimator_UnitFormat(unit)

	local name = UnitName(unit);
	
	if (name == "Dagma" and UnitClass(unit)=="Priest") then
        GameTooltip:AddLine("\"Beard flurry!\"");
    elseif (name == "Butterboots" and UnitClass(unit)=="Druid") then
        GameTooltip:AddLine("\"Everyone's personal healer.\"");
    elseif (name == "Rylea" and UnitClass(unit)=="Paladin") then
        GameTooltip:AddLine("\"Lovely assistant.\"");
    elseif (name == "Dorngard" and UnitClass(unit)=="Paladin") then
        GameTooltip:AddLine("\"Bubble's on cooldown!\"");
    elseif (name == "Serrac" and UnitClass(unit)=="Druid") then
        GameTooltip:AddLine("\"Weak heal incoming!\"");
	end
	GameTooltip:Show();
end

