HEALINGESTIMATOR_VERSION = "v1.3.5";

-- Localization strings
HealingEstimatorLoc = 
	{
	Welcome		= "HealingEstimator "..HEALINGESTIMATOR_VERSION.." loaded. /healest for options.";
	Welcome2   = "HealingEstimator text commands:";
	Welcome3   = "  /healest -- shows options dialog";
	Welcome4   = "  /healest portrait -- toggles heal target portrait";
	Welcome5   = "  /healest math -- add healing math to spell tooltips";
	Welcome6   = "  /healest glue -- makes healing bar stick to target (toggle)";
	Welcome7   = "  /healest buttontext -- toggle display of average heal on spell buttons";
	Welcome8   = "  /healest spellbonus Spell Name-### -- adds arbitrary bonus healing amount ### to all spells named Spell Name";
	
	-- This infomation is read from tooltip
	HealValue	= "(%d+) to (%d+)";
	RankTxt		= "%((Rank %d+)%)";
	RankTooltip	= "Rank (%d+)";
	TargetName	= "<Target name>";
	HOTValue   = "(%d+) over (%d+)";
	Casttime   = "(.+) sec cast";
	Instant    = "Instant cast";
	PoMHealValue = "for (%d+)";
	PWSAbsorb = "absorbing (%d+)";
	
	-- Healing texts in combat log
	CritHeal	= "Your (.+) critically heals (.+) for (%d+)%.";
	NormHeal	= "Your (.+) heals (.+) for (%d+)%.";
	HotSelf		= "You gain (%d+) health from (.+)%.";
	HotOther	= "(.+) gains (%d+) health from your (.+)%.";
	YouText 	= "you";
	
	-- Healing classes
	HealingClasses={"Priest", "Druid", "Paladin", "Shaman"};
	HealingSpells=
		{
		"Flash Heal", "Greater Heal", "Lesser Heal", "Heal", "Prayer of Mending","Binding Heal","Prayer of Healing","Power Word: Shield","Circle of Healing",
		"Healing Touch", "Regrowth",
		"Holy Light", "Flash of Light",
		"Healing Wave", "Lesser Healing Wave", "Chain Heal"
		};
	
	-- Menu text
	Scale			= "Healing bar scale ";
	Limit			= "Overheal warning at ";
	IconPos			= "Position";
	OverhealTitle	= "Healing meter config";
	BarTitle		= "Healing bar";
	MinimapTitle	= "Minimap button";
	ShowText		= "Show";
	HideText		= "Hide";
	ResetPosTooltip	= "Resets the healing bar position";
	HideTooltip		= "Hides/shows the minimap button";
	ClearTooltip	= "Clears current healing/overhealing data";
	
	
	HideMeter		= "Overhealing meter hidden, use /healest command to show menu.";
	ShowMeter		= "Overhealing meter visible.";
    };

HE_SPELL_FlashHeal = "Flash Heal";
HE_SPELL_Heal = "Heal";
HE_SPELL_GreaterHeal = "Greater Heal";
HE_SPELL_Renew = "Renew";
HE_SPELL_HealingTouch = "Healing Touch";
HE_SPELL_Regrowth = "Regrowth";
HE_SPELL_Rejuvenation = "Rejuvenation";
HE_SPELL_HolyLight = "Holy Light";
HE_SPELL_FlashOfLight = "Flash of Light";
HE_SPELL_PrayerOfMending = "Prayer of Mending";
HE_SPELL_BindingHeal = "Binding Heal";
HE_SPELL_PrayerOfHealing = "Prayer of Healing";
HE_SPELL_PowerWordShield = "Power Word: Shield";

HE_SPELL_Lifebloom = "Lifebloom";

HE_SPELL_HealingWave = "Healing Wave";
HE_SPELL_LesserHealingWave = "Lesser Healing Wave";
HE_SPELL_ChainHeal = "Chain Heal";

HE_SPELL_BlessingOfLight = "Blessing of Light";
HE_SPELL_GrBlessingOfLight = "Greater Blessing of Light";
HE_SPELL_FearWard = "Fear Ward";

HE_CL_DRUID = "DRUID";
HE_CL_HUNTER = "HUNTER";
HE_CL_MAGE = "MAGE";
HE_CL_PALADIN = "PALADIN";
HE_CL_PRIEST = "PRIEST";
HE_CL_ROGUE = "ROGUE";
HE_CL_SHAMAN = "SHAMAN";
HE_CL_WARLOCK = "WARLOCK";
HE_CL_WARRIOR = "WARRIOR";

--------------------------------------------------------------------
-- Localized functions
--------------------------------------------------------------------

--------------------------------------------------------------------
-- IsHealingSpell
-- Checks if given spell is a single target healing spell with cast time
--
function HealingEstimator_IsHealingSpell(Name)
	local c;
	local Size=table.getn(HealingEstimatorLoc.HealingSpells);
	for c=1,Size do
		if (HealingEstimatorLoc.HealingSpells[c]==Name) then
			-- Spell is healing spell, return true
			return true;
		end
	end
	-- Return false
	return false;	
end

--------------------------------------------------------------------
-- IsHealingClass
-- Checks if given class is capable of healing
--
function HealingEstimator_IsHealingClass(Class)
	local c;
	local Size=table.getn(HealingEstimatorLoc.HealingClasses);
	for c=1,Size do
		if (HealingEstimatorLoc.HealingClasses[c]==Class) then
			-- Spell is healing spell, return true
			return true;
		end
	end
	-- Return false
	return false;	
end

--------------------------------------------------------------------
-- ParseSelfBuff
-- Reads the heal info from combat log message
-- Returns Spell, Spell target, Heal amount, info about crit
--
function HealingEstimator_ParseSelfBuff(Msg)
	local s,t,h;
	local Spell,Target,Heal,Crit=nil,nil,nil,false;
	for s,t,h in string.gmatch(Msg, HealingEstimatorLoc.CritHeal) do
		Spell=s;Target=t;Heal=h;
		Crit=true;
	end
	if (not (Spell and Heal)) then
		for s,t,h in string.gmatch(Msg, HealingEstimatorLoc.NormHeal) do
			Spell=s;Target=t;Heal=h;
		end
	end
	
	-- Convert "you" to player name
	-- Seen in lines "Your Healing touch heals you for 42"
	if (Target and string.lower(Target)==HealingEstimatorLoc.YouText) then
		Target=UnitName("player");
	end

	return Spell,Target,Heal,Crit;
end

--------------------------------------------------------------------
-- ParsePeriodicSelfBuff
-- Reads the heal over time info from combat log message
-- Returns Spell, Spell target, Heal amount, info about crit
--
function HealingEstimator_ParsePeriodicSelfBuff(Msg)
	local Spell,Target,Heal;
	for Heal, Spell in string.gmatch(Msg, HealingEstimatorLoc.HotSelf) do 
		-- Self Hot
		return Spell,UnitName("player"),Heal,false; 
	end
	
	for Target, Heal, Spell in string.gmatch(Msg, HealingEstimatorLoc.HotOther) do 
		-- Self Hot on Others
		return Spell,Target,Heal,false; 
	end
	return nil,nil,nil,false;
end